@@ -32,8 +32,9 @@
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-vfs-extensions.h>
 #include <gtk/gtksignal.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs.h>
+#include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-trash-monitor.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <string.h>
@@ -117,6 +118,19 @@ trash_name_changed (gpointer callback_data)
 	nautilus_desktop_link_changed (link);
 }
 
+static void
+network_name_changed (gpointer callback_data)
+{
+	NautilusDesktopLink *link;
+
+	link = NAUTILUS_DESKTOP_LINK (callback_data);
+	g_assert (link->details->type == NAUTILUS_DESKTOP_LINK_NETWORK);
+
+	
+	g_free (link->details->display_name);
+	link->details->display_name = eel_preferences_get (NAUTILUS_PREFERENCES_DESKTOP_NETWORK_NAME);
+	nautilus_desktop_link_changed (link);
+}
 
 
 NautilusDesktopLink *
@@ -130,10 +144,8 @@ nautilus_desktop_link_new (NautilusDesktopLinkType type)
 	switch (type) {
 	case NAUTILUS_DESKTOP_LINK_HOME:
 		link->details->filename = g_strdup ("home");
-
 		link->details->display_name = eel_preferences_get (NAUTILUS_PREFERENCES_DESKTOP_HOME_NAME);
-		
-		link->details->activation_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
+		link->details->activation_uri = nautilus_get_home_directory_uri ();
 		link->details->icon = g_strdup ("gnome-fs-home");
 
 		eel_preferences_add_callback (NAUTILUS_PREFERENCES_DESKTOP_HOME_NAME,
@@ -177,9 +189,13 @@ nautilus_desktop_link_new (NautilusDesktopLinkType type)
 
 	case NAUTILUS_DESKTOP_LINK_NETWORK:
 		link->details->filename = g_strdup ("network");
-		link->details->display_name = _("Network Servers");
+		link->details->display_name = eel_preferences_get (NAUTILUS_PREFERENCES_DESKTOP_NETWORK_NAME);
 		link->details->activation_uri = g_strdup ("network:///");
 		link->details->icon = g_strdup ("gnome-fs-network");
+
+		eel_preferences_add_callback (NAUTILUS_PREFERENCES_DESKTOP_NETWORK_NAME,
+					      network_name_changed,
+					      link);
 		break;
 
 	default:
@@ -311,6 +327,7 @@ nautilus_desktop_link_can_rename (NautilusDesktopLink     *link)
 {
 	return (link->details->type == NAUTILUS_DESKTOP_LINK_HOME ||
 		link->details->type == NAUTILUS_DESKTOP_LINK_TRASH ||
+		link->details->type == NAUTILUS_DESKTOP_LINK_NETWORK ||
 		link->details->type == NAUTILUS_DESKTOP_LINK_COMPUTER);
 }
 
@@ -331,6 +348,10 @@ nautilus_desktop_link_rename (NautilusDesktopLink     *link,
 		eel_preferences_set (NAUTILUS_PREFERENCES_DESKTOP_TRASH_NAME,
 				     name);
 		break;
+	case NAUTILUS_DESKTOP_LINK_NETWORK:
+		eel_preferences_set (NAUTILUS_PREFERENCES_DESKTOP_NETWORK_NAME,
+				     name);
+		break;
 	default:
 		g_assert_not_reached ();
 		/* FIXME: Do we want volume renaming?
@@ -388,6 +409,12 @@ desktop_link_finalize (GObject *object)
 						 link);
 	}
 
+	if (link->details->type == NAUTILUS_DESKTOP_LINK_NETWORK) {
+		eel_preferences_remove_callback (NAUTILUS_PREFERENCES_DESKTOP_NETWORK_NAME,
+						 network_name_changed,
+						 link);
+	}
+
 	if (link->details->type == NAUTILUS_DESKTOP_LINK_VOLUME) {
 		gnome_vfs_volume_unref (link->details->volume);
 	}
