@@ -30,6 +30,7 @@
 #include "nautilus-global-preferences.h"
 #include "nautilus-icon-factory-private.h"
 #include "nautilus-icon-factory.h"
+#include <math.h>
 #include <eel/eel-gdk-pixbuf-extensions.h>
 #include <eel/eel-graphic-effects.h>
 #include <eel/eel-string.h>
@@ -86,15 +87,26 @@ static pthread_mutex_t thumbnails_mutex = PTHREAD_MUTEX_INITIALIZER;
    start more than one. Lock thumbnails_mutex when accessing this. */
 static volatile gboolean thumbnail_thread_is_running = FALSE;
 
+/* Added in glib 2.14 */
+#ifndef G_QUEUE_INIT
+#define G_QUEUE_INIT { NULL, NULL, 0 }
+#endif
+
 /* The list of NautilusThumbnailInfo structs containing information about the
    thumbnails we are making. Lock thumbnails_mutex when accessing this. */
-static volatile GList *thumbnails_to_make = NULL;
+static volatile GQueue thumbnails_to_make = G_QUEUE_INIT;
+
+/* Quickly check if uri is in thumbnails_to_make list */
+static GHashTable *thumbnails_to_make_hash = NULL;
+
 /* The currently thumbnailed icon. it also exists in the thumbnails_to_make list
  * to avoid adding it again. Lock thumbnails_mutex when accessing this. */
 static NautilusThumbnailInfo *currently_thumbnailing = NULL;
 
 static GnomeThumbnailFactory *thumbnail_factory = NULL;
 
+static int thumbnail_icon_size = 0;
+
 static gboolean
 get_file_mtime (const char *file_uri, time_t* mtime)
 {
@@ -114,21 +126,6 @@ get_file_mtime (const char *file_uri, time_t* mtime)
 	return TRUE;
 }
 
-/* GCompareFunc-style function for comparing NautilusThumbnailInfos.
- * Returns 0 if they refer to the same uri.
- */
-static int
-compare_thumbnail_info (gconstpointer a, gconstpointer b)
-{
-	NautilusThumbnailInfo *info_a;
-	NautilusThumbnailInfo *info_b;
-
-	info_a = (NautilusThumbnailInfo *)a;
-	info_b = (NautilusThumbnailInfo *)b;
-
-	return strcmp (info_a->image_uri, info_b->image_uri) != 0;
-}
-
 static void
 free_thumbnail_info (NautilusThumbnailInfo *info)
 {
@@ -178,27 +175,28 @@ thumbnail_thread_starter_cb (gpointer data)
 }
 
 void
-nautilus_update_thumbnail_file_renamed (const char *old_file_uri, const char *new_file_uri)
+nautilus_update_thumbnail_file_copied (const char *source_file_uri,
+				       const char *destination_file_uri)
 {
 	char *old_thumbnail_path;
 	GdkPixbuf *pixbuf;
 	GnomeVFSFileInfo *file_info;
 	GnomeThumbnailFactory *factory;
 	
-	old_thumbnail_path = gnome_thumbnail_path_for_uri (old_file_uri, GNOME_THUMBNAIL_SIZE_NORMAL);
+	old_thumbnail_path = gnome_thumbnail_path_for_uri (source_file_uri, GNOME_THUMBNAIL_SIZE_NORMAL);
 	if (old_thumbnail_path != NULL &&
 	    g_file_test (old_thumbnail_path, G_FILE_TEST_EXISTS)) {
 		file_info = gnome_vfs_file_info_new ();
-		if (gnome_vfs_get_file_info (new_file_uri,
+		if (gnome_vfs_get_file_info (destination_file_uri,
 					     file_info,
 					     GNOME_VFS_FILE_INFO_DEFAULT) == GNOME_VFS_OK) {
 			pixbuf = gdk_pixbuf_new_from_file (old_thumbnail_path, NULL);
 			
-			if (pixbuf && gnome_thumbnail_has_uri (pixbuf, old_file_uri)) {
+			if (pixbuf && gnome_thumbnail_has_uri (pixbuf, source_file_uri)) {
 				factory = nautilus_icon_factory_get_thumbnail_factory ();
 				gnome_thumbnail_factory_save_thumbnail (factory,
 									pixbuf,
-									new_file_uri,
+									destination_file_uri,
 									file_info->mtime);
 				g_object_unref (factory);
 			}
@@ -207,7 +205,6 @@ nautilus_update_thumbnail_file_renamed (const char *old_file_uri, const char *ne
 				g_object_unref (pixbuf);
 			}
 			
-			unlink (old_thumbnail_path);
 		}
 		gnome_vfs_file_info_unref (file_info);
 	}
@@ -215,16 +212,24 @@ nautilus_update_thumbnail_file_renamed (const char *old_file_uri, const char *ne
 	g_free (old_thumbnail_path);
 }
 
-void 
-nautilus_remove_thumbnail_for_file (const char *old_file_uri)
+void
+nautilus_update_thumbnail_file_renamed (const char *source_file_uri,
+					const char *destination_file_uri)
 {
-	char *old_thumbnail_path;
+	nautilus_update_thumbnail_file_copied (source_file_uri, destination_file_uri);
+	nautilus_remove_thumbnail_for_file (source_file_uri);
+}
+
+void 
+nautilus_remove_thumbnail_for_file (const char *file_uri)
+{
+	char *thumbnail_path;
 	
-	old_thumbnail_path = gnome_thumbnail_path_for_uri (old_file_uri, GNOME_THUMBNAIL_SIZE_NORMAL);
-	if (old_thumbnail_path != NULL) {
-		unlink (old_thumbnail_path);
+	thumbnail_path = gnome_thumbnail_path_for_uri (file_uri, GNOME_THUMBNAIL_SIZE_NORMAL);
+	if (thumbnail_path != NULL) {
+		unlink (thumbnail_path);
 	}
-	g_free (old_thumbnail_path);
+	g_free (thumbnail_path);
 }
 
 void
@@ -255,26 +260,158 @@ nautilus_thumbnail_frame_image (GdkPixbuf **pixbuf)
 	*pixbuf=pixbuf_with_frame;
 }
 
-/* routine to load an image from the passed-in path, and then embed it in
- * a frame if necessary
- */
-GdkPixbuf *
-nautilus_thumbnail_load_framed_image (const char *path)
+typedef struct {
+	gboolean is_thumbnail;
+	guint base_size;
+	guint nominal_size;
+	gboolean force_nominal;
+	int original_height;
+	int original_width;
+	double *scale_x_out;
+	double *scale_y_out;
+} ThumbnailLoadArgs;
+
+static void
+thumbnail_loader_size_prepared (GdkPixbufLoader *loader,
+				int width,
+				int height,
+				ThumbnailLoadArgs *args)
+{
+	int size = MAX (width, height);
+
+	args->original_width = width;
+	args->original_height = height;
+
+	if (args->force_nominal) {
+		args->base_size = size;                        
+	} else if (args->base_size == 0) {
+		if (args->is_thumbnail) {
+			args->base_size = 128 * NAUTILUS_ICON_SIZE_STANDARD / thumbnail_icon_size;
+		} else {
+			if (size > args->nominal_size * thumbnail_icon_size / NAUTILUS_ICON_SIZE_STANDARD) {
+				args->base_size = size * NAUTILUS_ICON_SIZE_STANDARD / thumbnail_icon_size;
+			} else if (size > NAUTILUS_ICON_SIZE_STANDARD) {
+				args->base_size = args->nominal_size;
+			} else {
+				/* Don't scale up small icons */
+				args->base_size = NAUTILUS_ICON_SIZE_STANDARD;
+			}
+		}
+	}
+
+	if (args->base_size != args->nominal_size) {
+		double scale;
+
+		scale = (double) args->nominal_size / args->base_size;
+
+		if ((int) (width * scale) > NAUTILUS_ICON_MAXIMUM_SIZE ||
+		    (int) (height * scale) > NAUTILUS_ICON_MAXIMUM_SIZE) {
+			scale = MIN ((double) NAUTILUS_ICON_MAXIMUM_SIZE / width,
+				     (double) NAUTILUS_ICON_MAXIMUM_SIZE / height);
+		}
+
+		width = MAX (1, floor (width * scale + 0.5));
+		height = MAX (1, floor (height * scale + 0.5));
+
+		gdk_pixbuf_loader_set_size (loader, width, height);
+	}
+
+}
+
+static void
+thumbnail_loader_area_prepared (GdkPixbufLoader *loader,
+				ThumbnailLoadArgs *args)
 {
 	GdkPixbuf *pixbuf;
+
+	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
+
+	*args->scale_x_out = (double) gdk_pixbuf_get_width (pixbuf) / args->original_width;
+	*args->scale_y_out = (double) gdk_pixbuf_get_height (pixbuf) / args->original_height;
+}
+
+/* routine to load an image from the passed-in path
+ */
+GdkPixbuf *
+nautilus_thumbnail_load_image (const char *path,
+			       guint base_size,
+			       guint nominal_size,
+			       gboolean force_nominal,
+			       double *scale_x_out,
+			       double *scale_y_out)
+{
+	guchar *buffer;
+	GdkPixbufLoader *loader;
+	GdkPixbuf *pixbuf;
+	GError *error;
+	gsize buflen;
+	ThumbnailLoadArgs args;
+
+	error = NULL;
+
+	if (thumbnail_icon_size == 0) {
+		eel_preferences_add_auto_integer (NAUTILUS_PREFERENCES_ICON_VIEW_THUMBNAIL_SIZE,
+						  &thumbnail_icon_size);
+	}
+
 	
-	pixbuf = gdk_pixbuf_new_from_file (path, NULL);
-	if (pixbuf == NULL) {
+	if (!g_file_get_contents (path, (gchar **) &buffer, &buflen, &error)) {
+		g_message ("Failed to load %s into memory: %s", path, error->message);
+
+		g_error_free (error);
+
 		return NULL;
 	}
-        nautilus_thumbnail_frame_image(&pixbuf);	
-        return pixbuf;
+
+	loader = gdk_pixbuf_loader_new ();
+	g_signal_connect (loader, "size-prepared",
+			  G_CALLBACK (thumbnail_loader_size_prepared),
+			  &args);
+	g_signal_connect (loader, "area-prepared",
+			  G_CALLBACK (thumbnail_loader_area_prepared),
+			  &args);
+
+	args.is_thumbnail = strstr (path, "/.thumbnails/") != NULL;
+	args.base_size = base_size;
+	args.nominal_size = nominal_size;
+	args.force_nominal = force_nominal;
+	args.scale_x_out = scale_x_out;
+	args.scale_y_out = scale_y_out;
+
+	if (!gdk_pixbuf_loader_write (loader, buffer, buflen, &error)) {
+		g_message ("Failed to write %s to thumbnail pixbuf loader: %s", path, error->message);
+
+		gdk_pixbuf_loader_close (loader, NULL);
+		g_object_unref (G_OBJECT (loader));
+		g_error_free (error);
+		g_free (buffer);
+
+		return NULL;
+	}
+
+	if (!gdk_pixbuf_loader_close (loader, &error) ||
+	    /* Seems we have to check this even if it returned TRUE (#403255) */
+	    error != NULL) {
+		g_message ("Failed to close thumbnail pixbuf loader for %s: %s", path, error->message);
+
+		g_object_unref (G_OBJECT (loader));
+		g_error_free (error);
+		g_free (buffer);
+
+		return NULL;
+	}
+
+	pixbuf = g_object_ref (gdk_pixbuf_loader_get_pixbuf (loader));
+
+	g_object_unref (G_OBJECT (loader));
+	g_free (buffer);
+
+	return pixbuf;
 }
 
 void
 nautilus_thumbnail_remove_from_queue (const char *file_uri)
 {
-	NautilusThumbnailInfo info;
 	GList *node;
 	
 #ifdef DEBUG_THUMBNAILS
@@ -286,15 +423,14 @@ nautilus_thumbnail_remove_from_queue (const char *file_uri)
 	 * MUTEX LOCKED
 	 *********************************/
 
-	info.image_uri = (char *)file_uri;
-	info.mime_type = NULL;
-	
-	node = g_list_find_custom ((GList*) thumbnails_to_make, &info,
-				   compare_thumbnail_info);
-
-	if (node && node->data != currently_thumbnailing) {
-		free_thumbnail_info (node->data);
-		thumbnails_to_make = g_list_delete_link ((GList *)thumbnails_to_make, node);
+	if (thumbnails_to_make_hash) {
+		node = g_hash_table_lookup (thumbnails_to_make_hash, file_uri);
+		
+		if (node && node->data != currently_thumbnailing) {
+			g_hash_table_remove (thumbnails_to_make_hash, file_uri);
+			free_thumbnail_info (node->data);
+			g_queue_delete_link ((GQueue *)&thumbnails_to_make, node);
+		}
 	}
 	
 	/*********************************
@@ -310,6 +446,7 @@ nautilus_thumbnail_remove_from_queue (const char *file_uri)
 void
 nautilus_thumbnail_remove_all_from_queue (void)
 {
+	NautilusThumbnailInfo *info;
 	GList *l, *next;
 	
 #ifdef DEBUG_THUMBNAILS
@@ -321,12 +458,15 @@ nautilus_thumbnail_remove_all_from_queue (void)
 	 * MUTEX LOCKED
 	 *********************************/
 
-	l = (GList *)thumbnails_to_make;
+	l = thumbnails_to_make.head;
 	while (l != NULL) {
+		info = l->data;
 		next = l->next;
-		if (l->data != currently_thumbnailing) {
-			free_thumbnail_info (l->data);
-			thumbnails_to_make = g_list_delete_link ((GList *)thumbnails_to_make, l);
+		if (info != currently_thumbnailing) {
+			g_hash_table_remove (thumbnails_to_make_hash, 
+					     info->image_uri);
+			free_thumbnail_info (info);
+			g_queue_delete_link ((GQueue *)&thumbnails_to_make, l);
 		}
 
 		l = next;
@@ -345,7 +485,6 @@ nautilus_thumbnail_remove_all_from_queue (void)
 void
 nautilus_thumbnail_prioritize (const char *file_uri)
 {
-	NautilusThumbnailInfo info;
 	GList *node;
 
 #ifdef DEBUG_THUMBNAILS
@@ -357,15 +496,13 @@ nautilus_thumbnail_prioritize (const char *file_uri)
 	 * MUTEX LOCKED
 	 *********************************/
 
-	info.image_uri = (char *)file_uri;
-	info.mime_type = NULL;
-	
-	node = g_list_find_custom ((GList*) thumbnails_to_make, &info,
-				   compare_thumbnail_info);
-
-	if (node && node->data != currently_thumbnailing) {
-		thumbnails_to_make = g_list_remove_link ((GList *)thumbnails_to_make, node);
-		thumbnails_to_make = g_list_concat (node, (GList *)thumbnails_to_make);
+	if (thumbnails_to_make_hash) {
+		node = g_hash_table_lookup (thumbnails_to_make_hash, file_uri);
+		
+		if (node && node->data != currently_thumbnailing) {
+			g_queue_unlink ((GQueue *)&thumbnails_to_make, node);
+			g_queue_push_head_link ((GQueue *)&thumbnails_to_make, node);
+		}
 	}
 	
 	/*********************************
@@ -418,7 +555,7 @@ nautilus_create_thumbnail (NautilusFile *file)
 	time_t file_mtime = 0;
 	NautilusThumbnailInfo *info;
 	NautilusThumbnailInfo *existing_info;
-	GList *existing;
+	GList *existing, *node;
 
 	nautilus_file_set_is_thumbnailing (file, TRUE);
 
@@ -443,28 +580,35 @@ nautilus_create_thumbnail (NautilusFile *file)
 	g_message ("(Main Thread) Locking mutex\n");
 #endif
 	pthread_mutex_lock (&thumbnails_mutex);
-
+	
 	/*********************************
 	 * MUTEX LOCKED
 	 *********************************/
 
+	if (thumbnails_to_make_hash == NULL) {
+		thumbnails_to_make_hash = g_hash_table_new (g_str_hash,
+							    g_str_equal);
+	}
+
 	/* Check if it is already in the list of thumbnails to make. */
-	existing = g_list_find_custom ((GList*) thumbnails_to_make, info,
-				       compare_thumbnail_info);
+	existing = g_hash_table_lookup (thumbnails_to_make_hash, info->image_uri);
 	if (existing == NULL) {
 		/* Add the thumbnail to the list. */
 #ifdef DEBUG_THUMBNAILS
 		g_message ("(Main Thread) Adding thumbnail: %s\n",
 			   info->image_uri);
 #endif
-		thumbnails_to_make = g_list_append ((GList*) thumbnails_to_make, info);
-		
+		g_queue_push_tail ((GQueue *)&thumbnails_to_make, info);
+		node = g_queue_peek_tail_link ((GQueue *)&thumbnails_to_make);
+		g_hash_table_insert (thumbnails_to_make_hash,
+				     info->image_uri,
+				     node);
 		/* If the thumbnail thread isn't running, and we haven't
 		   scheduled an idle function to start it up, do that now.
 		   We don't want to start it until all the other work is done,
 		   so the GUI will be updated as quickly as possible.*/
-		if (thumbnail_thread_is_running == FALSE
-		    && thumbnail_thread_starter_id == 0) {
+		if (thumbnail_thread_is_running == FALSE &&
+		    thumbnail_thread_starter_id == 0) {
 			thumbnail_thread_starter_id = g_idle_add_full (G_PRIORITY_LOW, thumbnail_thread_starter_cb, NULL, NULL);
 		}
 	} else {
@@ -476,8 +620,7 @@ nautilus_create_thumbnail (NautilusFile *file)
 		existing_info = existing->data;
 		existing_info->original_file_mtime = info->original_file_mtime;
 		free_thumbnail_info (info);
-	}
-	    
+	}   
 
 	/*********************************
 	 * MUTEX UNLOCKED
@@ -497,6 +640,7 @@ thumbnail_thread_start (gpointer data)
 	GdkPixbuf *pixbuf;
 	time_t current_orig_mtime = 0;
 	time_t current_time;
+	GList *node;
 
 	/* We loop until there are no more thumbails to make, at which point
 	   we exit the thread. */
@@ -520,15 +664,18 @@ thumbnail_thread_start (gpointer data)
 		if (currently_thumbnailing &&
 		    currently_thumbnailing->original_file_mtime == current_orig_mtime) {
 			g_assert (info == currently_thumbnailing);
-			free_thumbnail_info (currently_thumbnailing);
-			thumbnails_to_make = g_list_remove ((GList*) thumbnails_to_make, currently_thumbnailing);
+			node = g_hash_table_lookup (thumbnails_to_make_hash, info->image_uri);
+			g_assert (node != NULL);
+			g_hash_table_remove (thumbnails_to_make_hash, info->image_uri);
+			free_thumbnail_info (info);
+			g_queue_delete_link ((GQueue *)&thumbnails_to_make, node);
 		}
 		currently_thumbnailing = NULL;
 
 		/* If there are no more thumbnails to make, reset the
 		   thumbnail_thread_is_running flag, unlock the mutex, and
 		   exit the thread. */
-		if (thumbnails_to_make == NULL) {
+		if (g_queue_is_empty ((GQueue *)&thumbnails_to_make)) {
 #ifdef DEBUG_THUMBNAILS
 			g_message ("(Thumbnail Thread) Exiting\n");
 #endif
@@ -540,7 +687,7 @@ thumbnail_thread_start (gpointer data)
 		/* Get the next one to make. We leave it on the list until it
 		   is created so the main thread doesn't add it again while we
 		   are creating it. */
-		info = thumbnails_to_make->data;
+		info = g_queue_peek_head ((GQueue *)&thumbnails_to_make);
 		currently_thumbnailing = info;
 		current_orig_mtime = info->original_file_mtime;
 		/*********************************
