@@ -36,7 +36,7 @@
 #include <gtk/gtkstock.h>
 #include <gtk/gtkdnd.h>
 #include <gtk/gtkfilechooserdialog.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomeui/gnome-uidefs.h>
 #include <libnautilus-extension/nautilus-extension-types.h>
 #include <libnautilus-extension/nautilus-file-info.h>
@@ -63,7 +63,7 @@ enum {
 	TARGET_URI_LIST
 };
 
-static GtkTargetEntry target_table[] = {
+static const GtkTargetEntry target_table[] = {
         { "text/uri-list",  0, TARGET_URI_LIST }
 };
 
@@ -470,7 +470,7 @@ fm_ditem_page_get_type (void)
 	static GType provider_type = 0;
 
 	if (!provider_type) {
-		static const GTypeInfo type_info = {
+		const GTypeInfo type_info = {
 			sizeof (FMDitemPageClass),
 			(GBaseInitFunc) NULL,
 			(GBaseFinalizeFunc) NULL,
@@ -482,7 +482,7 @@ fm_ditem_page_get_type (void)
 			(GInstanceInitFunc) fm_ditem_page_instance_init,
 		};
 
-		static const GInterfaceInfo property_page_provider_iface_info = {
+		const GInterfaceInfo property_page_provider_iface_info = {
 			(GInterfaceInitFunc) fm_ditem_page_property_page_provider_iface_init,
 			NULL,
 			NULL
