@@ -36,7 +36,6 @@
 #include "nautilus-bookmarks-window.h"
 #include "nautilus-file-management-properties.h"
 #include "nautilus-property-browser.h"
-#include "nautilus-signaller.h"
 #include "nautilus-window-manage-views.h"
 #include "nautilus-window-private.h"
 #include "nautilus-window-bookmarks.h"
@@ -49,7 +48,7 @@
 #include <libxml/parser.h>
 #include <gtk/gtkmain.h>
 #include <libgnome/gnome-help.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-about.h>
 #include <libgnomeui/gnome-uidefs.h>
@@ -60,10 +59,13 @@
 #include <libnautilus-private/nautilus-ui-utilities.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-undo-manager.h>
+#include <libnautilus-private/nautilus-search-engine.h>
+#include <libnautilus-private/nautilus-signaller.h>
 
 #define MENU_PATH_HISTORY_PLACEHOLDER			"/MenuBar/Other Menus/Go/History Placeholder"
 
 #define RESPONSE_FORGET		1000
+#define MENU_ITEM_MAX_WIDTH_CHARS 32
 
 static void                  schedule_refresh_go_menu                      (NautilusNavigationWindow   *window);
 
@@ -123,7 +125,6 @@ forget_history_if_confirmed (NautilusWindow *window)
 					   
 	dialog = eel_create_question_dialog (prompt,
 					     detail,
-					     _("Clear History"), 
 					     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					     GTK_STOCK_CLEAR, RESPONSE_FORGET,
 					     GTK_WINDOW (window));
@@ -289,7 +290,6 @@ show_bogus_history_window (NautilusWindow *window,
 
 	eel_show_warning_dialog (_("The history location doesn't exist."),
 				 detail,
-				 _("Bookmark for Nonexistent Location"),
 				 GTK_WINDOW (window));
 
 	g_free (uri);
@@ -297,6 +297,23 @@ show_bogus_history_window (NautilusWindow *window,
 	g_free (detail);
 }
 
+static void
+connect_proxy_cb (GtkActionGroup *action_group,
+                  GtkAction *action,
+                  GtkWidget *proxy,
+                  gpointer dummy)
+{
+	GtkLabel *label;
+
+	if (!GTK_IS_MENU_ITEM (proxy))
+		return;
+
+	label = GTK_LABEL (GTK_BIN (proxy)->child);
+
+	gtk_label_set_use_underline (label, FALSE);
+	gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_END);
+	gtk_label_set_max_width_chars (label, MENU_ITEM_MAX_WIDTH_CHARS);
+}
 
 /**
  * refresh_go_menu:
@@ -323,6 +340,8 @@ refresh_go_menu (NautilusNavigationWindow *window)
 
 	window->details->go_menu_merge_id = gtk_ui_manager_new_merge_id (ui_manager);
 	window->details->go_menu_action_group = gtk_action_group_new ("GoMenuGroup");
+	g_signal_connect (window->details->go_menu_action_group, "connect-proxy",
+			  G_CALLBACK (connect_proxy_cb), NULL);
 
 	gtk_ui_manager_insert_action_group (ui_manager,
 					    window->details->go_menu_action_group,
@@ -412,10 +431,21 @@ action_go_to_location_callback (GtkAction *action,
 	nautilus_window_prompt_for_location (window, NULL);
 }			   
 
+static void
+action_search_callback (GtkAction *action,
+			gpointer user_data)
+{
+	NautilusNavigationWindow *window;
+
+	window = NAUTILUS_NAVIGATION_WINDOW (user_data);
+
+	nautilus_navigation_window_show_search (window);
+}
+
 static const GtkActionEntry navigation_entries[] = {
   { "Go", NULL, N_("_Go") },               /* name, stock id, label */
   { "Bookmarks", NULL, N_("_Bookmarks") },               /* name, stock id, label */
-  { "New Window", NULL, N_("Open New _Window"),               /* name, stock id, label */
+  { "New Window", "window-new", N_("Open New _Window"),               /* name, stock id, label */
     "<control>N", N_("Open another Nautilus window for the displayed location"),
     G_CALLBACK (action_new_window_callback) },
   { "Close All Windows", NULL, N_("Close _All Windows"),               /* name, stock id, label */
@@ -433,6 +463,10 @@ static const GtkActionEntry navigation_entries[] = {
   { "Edit Bookmarks", NULL, N_("_Edit Bookmarks"), /* name, stock id, label */
     "<control>b", N_("Display a window that allows editing the bookmarks in this menu"),
     G_CALLBACK (action_edit_bookmarks_callback) },
+  { "Search", "gtk-find", N_("_Search for Files..."), /* name, stock id, label */
+    "<control>F", N_("Locate documents and folders on this computer by name or content"),
+    G_CALLBACK (action_search_callback) },
+		     
 };
 
 static const GtkToggleActionEntry navigation_toggle_entries[] = {
@@ -507,8 +541,12 @@ nautilus_navigation_window_initialize_actions (NautilusNavigationWindow *window)
 	gtk_action_group_add_action_with_accel (action_group,
 						action,
 						"<alt>Right");
+
 	g_object_unref (action);
 
+	action = gtk_action_group_get_action (action_group, NAUTILUS_ACTION_SEARCH);
+	g_object_set (action, "short_label", _("_Search"), NULL);
+
 	ui_manager = nautilus_window_get_ui_manager (NAUTILUS_WINDOW (window));
 
 	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
@@ -527,7 +565,7 @@ nautilus_navigation_window_initialize_menus (NautilusNavigationWindow *window)
 {
 	GtkUIManager *ui_manager;
 	const char *ui;
-	
+
 	ui_manager = nautilus_window_get_ui_manager (NAUTILUS_WINDOW (window));
 
 	ui = nautilus_ui_string_get ("nautilus-navigation-window-ui.xml");
