@@ -89,7 +89,7 @@ nautilus_search_bar_class_init (NautilusSearchBarClass *class)
 	signals[CANCEL] =
 		g_signal_new ("cancel",
 			      G_TYPE_FROM_CLASS (class),
-			      G_SIGNAL_RUN_LAST | GTK_RUN_ACTION,
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 			      G_STRUCT_OFFSET (NautilusSearchBarClass, cancel),
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
@@ -138,8 +138,7 @@ nautilus_search_bar_init (NautilusSearchBar *bar)
 	gtk_widget_show (hbox);
 	gtk_container_add (GTK_CONTAINER (alignment), hbox);
 
-	label = gtk_label_new ("");
-	gtk_label_set_markup (GTK_LABEL (label), _("<b>Search:</b>"));
+	label = gtk_label_new (_("Search:"));
 	gtk_widget_show (label);
 
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
